import os
from os.path import join
import sys
import os
import random
import configparser
from time import time

# COMPSs/PyCOMPSs imports
from pycompss.api.task import task
from pycompss.api.binary import binary
from pycompss.api.constraint import constraint
from pycompss.api.mpi import mpi
from pycompss.api.api import compss_wait_on
from pycompss.api.api import compss_barrier
from pycompss.api.parameter import *



# date
# mpirun -n 288 fesom.x > fesom2_log.out
# date

# TODO: check how to properly configure the JSON file, also which version of COMPSs supports these new decorators
#@Software(esm.json)
@mpi(binary="${FESOM_WORKINGDIR}/${FESOM_EXE}", runner="mpirun", processes="${FESOM_CORES}", working_dir="${FESOM_WORKINGDIR}", fail_by_exit_value=True)
@task(log_file={Type:FILE_OUT, StdIOStream:STDOUT})
def esm_simulation(log_file):
   pass

def esm_pruning():
   # 1 - connect to hecuba and obtain values

   # 2 - for each simulation to be stopped, remove it from the task group

   # 3 - remove its data from hecuba and remove its working directory and generated outputs

# takes the ensemble config file and prepares folders and config files
# TODO: this should be carry out by DLS ?
def esm_ensemble_init():

   # 1 - load ensemble config file
   config = configparser.ConfigParser()
   config.read(os.path.join(os.path.dirname(__file__), 'config', 'esm_ensemble.conf'))
   # 2 - create folders for the topdir and each ensemble member runtime folder
   if config.has_option('common', 'top_working_dir'):
      top_dir = config['common']['top_working_dir']
      output_dir = config['common']['top_working_dir']
      # define the access rights - readable and accessible by all users, and write access by only the owner
      access_rights = 0o755
      try:
         # top dir
         os.mkdir(top_dir , access_rights)
         # output dir
         os.mkdir(output_dir, access_rights)
         # get list of members
         sdates_list = (config['common']['ensemble_start_dates']).split()
         for sdate in sdates_list:
            os.mkdir(top_dir + "/" + sdate, access_rights)
            os.mkdir(output_dir + "/" + sdate, access_rights)
            #TODO: create function to process the namelist templates passing the config
            #link FESOM executable
            fesom_x = config['fesom2']['fesom_binary_path']
            os.symlink(fesom_x, output_dir + "/" + sdate)


      except OSError:
         print("Creation of the directory %s failed" % topdir)
   else:
      print("ERROR - topdir undefined path not found")

   # 3 - prepare config files for each simulation: replace placeholders in namelists for each member of the ensemble (start date), create fesom.clock. etc

# function to run ESM ensemble, each one should have different start dates or initial conditions/ forcings
# for making sense
def esm_ensemble(exp_id):

   # 1 - Creation of group
   with TaskGroup(str(exp_id), False):
      #2 - launch Analysis agent for the pruning

      #3 - Launch the ensemble members
      # TODO: get nr of members based on the list of dates defined in the ensemble config
      for mb in range( 1, int(members)+1 ):
         log = "fesom2_" + str(exp_id) + "_" + str(mb) + ".out"
         esm_simulation(log)

   compss_barrier_group(str(exp_id))


if __name__ == "__main__":
   print("Running FESOM2 - core2 dataset")
   # first parameter: number of ensemble members
   esm_members = sys.argv[1]
   # 0 - create a ramdon nr for the experiment id
   exp_id = random.randint(100000, 999999)
   esm_ensemble_init(exp_id)
   print("Running ESM Ensemble of " + str(esm_members))
   esm_ensemble(exp_id)

